#! /usr/bin/python3

import re

REGEX_VALID_CHAR = re.compile('[-_|a-zA-Z0-9/@]')
REGEX_LINE_INVALID = re.compile('(^#|^$)')
REGEX_RULE_DESCRIPTION = re.compile('^(header|body|score|include)')

def parse_line(line):
  result = ["", "", ""]
  i = 0
  while REGEX_VALID_CHAR.match(line[i]):
    result[0] += line[i]
    i += 1
  while not REGEX_VALID_CHAR.match(line[i]):
    i += 1
  while REGEX_VALID_CHAR.match(line[i]):
    result[1] += line[i]
    i += 1
  while i < len(line) and not REGEX_VALID_CHAR.match(line[i]):
    i += 1
  while i < len(line):
    result[2] += line[i]
    i += 1
  result[2] = result[2].strip()
  return (result[0], result[1], result[2])

def parse(rules, filepath):
  print("Parsing file", filepath)
  ifstream = open(filepath, 'r', encoding='utf_8')
  for line in ifstream:
    line = line.strip()
    if REGEX_LINE_INVALID.match(line):
      continue
    parsed_line = parse_line(line)
    if parsed_line[0] == "header" or parsed_line[0] == "body":
      rule = rules.get_rule(parsed_line[1])
      rule.name = parsed_line[1]
      if parsed_line[0] == "header":
        regex_split = parsed_line[2].split('=~')
        rule.header = regex_split[0].strip(': ')
        rule.regex = regex_split[1].strip(': ')
      else:
        rule.regex = parsed_line[2].strip()
    elif parsed_line[0] == "score":
      rules.get_rule(parsed_line[1]).score = parsed_line[2].strip()
    elif parsed_line[0] == "include":
      parse(rules, '/'.join(filepath.split('/')[0:-1]) + '/' + line.split(' ')[1])
  ifstream.close()

