#! /usr/bin/python3

import re
import config

REGEX_HTTP = re.compile(r'http://.*\{body\}')

def write_rules(rules):
  ofstream_metrics = open('/etc/rspamd/override.d/metrics.conf', 'w', encoding='utf_8')
  ofstream_rules = open('/etc/rspamd/rspamd.local.lua', 'w', encoding='utf_8')
  ofstream_metrics.write(config.METRICS_HEAD)
  ofstream_rules.write(config.RULES_HEAD)
  rule = rules.pop_rule()
  while rule != None:
    if rule.name in config.RULE_BLACKLIST or REGEX_HTTP.match(rule.regex):
      rule = rules.pop_rule()
      continue
    rule.regex = escape_regex(rule.regex.strip())
    ofstream_metrics.write('symbol "%s" {\n  score = %s;\n}\n' % (rule.name, rule.score))
    ofstream_rules.write(form_rule(rule))
    rule = rules.pop_rule()
  ofstream_metrics.close()
  ofstream_rules.close()

def form_rule(rule):
  if rule.header:
    return "config['regexp']['%s'] = '%s=%s'\n" % (rule.name, rule.header, rule.regex)
  else:
    return "config['regexp']['%s'] = '%s{body}'\n" % (rule.name, rule.regex)

def escape_regex(regex):
  regex = regex.strip() \
    .replace(r'\.', '.') \
    .replace(r'\ ', ' ') \
    .replace(r'\@', '@') \
    .replace(r'\-', '-') \
    .replace(r'\&', '&') \
    .replace(r'\~', '~') \
    .replace(r'\ä', 'ä') \
    .replace(r'\ö', 'ö') \
    .replace(r'\;', ';') \
    .replace(r'\°', '°') \
    .replace(r'\€', '€') \
    .replace(r'\<', '<') \
    .replace(r'\>', '>') \
    .replace(r'\*', '[*]') \
    .replace(r'\?', '[?]') \
    .replace(r'\"', '"') \
    .replace(r'\=', '=') \
    .replace(r'\!', '!') \
    .replace(r'\:', '[:]') \
    .replace(r'\[', '[') \
    .replace(r'\]', ']') \
    .replace(r'\%', '%') \
    .replace(r"'", r"\'").replace(r"\\'", r"\'") \
    .replace(r'\$', '$') \
    .replace(r'\,', ',') \
    .replace(r'\s', 's') \
    .replace(r'\c', 'c') \
    .replace(r'\(', '[(]') \
    .replace(r'\)', '[)]') \
    .replace(r'\+', '[+]') \
    .replace(r'\â', 'â') \
    .replace(r'\_', '_') \
    .replace(r'\/', '/')
  return regex

